@@ -204,7 +204,7 @@ main_dump_info()
 #endif
     log_info("------------------------------------------------");
     log_info("YADIFA is maintained by EURid");
-    log_info("Source code is available at http://www.yadifa.eu");
+    log_info("Source code is available at https://www.yadifa.eu");
     log_info("------------------------------------------------");
     log_info("got %u CPUs", sys_get_cpu_count());
     log_info("using %u UDP listeners per interface", g_config->thread_count_by_address);
@@ -212,6 +212,9 @@ main_dump_info()
 #if DNSCORE_HAS_ZALLOC_SUPPORT
     log_info("self-managed memory enabled"); // ZALLOC
 #endif
+#if HAS_RRL_SUPPORT
+    log_info("response rate limiter enabled"); // RRL
+#endif
 }
 
 static bool yadifad_config_on_section_loggers_read_done_once = FALSE;
